import math
import imp
imp.load_source("util","ScriptEditor/util.py")  
from util import *

translator.install()

def psat(t):
	if t >= 0.0:
		return 610.5 * math.exp((17.269 * t) / (237.3 + t))
	else:
		return 610.5 * math.exp((21.875 * t) / (265.5 + t))

def getTFAnalysisDimensionBind(analysis):
	if "length" in analysis:
		return "length"
	if "area" in analysis:
		return "area"
	return "length"

# Generate boundary groups names from a list of boundary groups
def generateBoundaryGroupsNames(boundaryGroups, boundaryConditions, groupBind = "group"):
	boundaryGroupNames = list()
	for boundaryGroup in boundaryGroups:
		groupName = ""
		for boundary in boundaryGroup:
			if groupBind in boundaryConditions[boundary]:
				groupID = boundaryConditions[boundary][groupBind]
			else:
				groupID = ""
			if isinstance(groupID, str) and groupID and not groupID.isdigit():
				groupName = groupID
				#print(groupID)
				break;

			if groupName != "":
				groupName = groupName + "\n"
			groupName = groupName + boundaryConditions[boundary]["name"]
		boundaryGroupNames.append(groupName)
	return boundaryGroupNames
	
# Count how many boundaries are begin used
def countUsedBoundaries(boundaryTFAnalysis):
	numUsedBoundaries = 0
	for currentBoundaryTFAnalysis in boundaryTFAnalysis:
		if currentBoundaryTFAnalysis[getTFAnalysisDimensionBind(currentBoundaryTFAnalysis)] > 0.0 and currentBoundaryTFAnalysis["type"] != "adiabatic":
			numUsedBoundaries = numUsedBoundaries + 1
	return numUsedBoundaries
	
def createBoundaryGroupTFAnalysisDescriptor(bind, boundaryVariant, simulation_3d):
	# Add group analysis only if some boundary is grouped
	if simulation_3d:
		dimensionBind = "area"
		dimensionName = _("Area")
		dimensionUnit = "m²"
		dimensionFormat = "%.4f"
		dimensionGridFormat = "%.2f"
		flowUnit = "W"
	else:
		dimensionBind = "length"
		dimensionName = _("Length")	
		dimensionUnit = "mm"
		dimensionFormat = "%.2f"
		dimensionGridFormat = "%.0f"
		flowUnit = "W/m"
	
	boundaryGroupTFAnalysisDescriptor = createEmptyGridDescriptor(bind, _("Grouped temperature and flow analysis"), "gtfan", 600, -1)
	addGridColStringDescriptor(boundaryGroupTFAnalysisDescriptor, "name", _("Name"))
	addGridColNumberDescriptor(boundaryGroupTFAnalysisDescriptor, dimensionBind, dimensionName,  dimensionUnit, dimensionFormat)
	addGridColNumberDescriptor(boundaryGroupTFAnalysisDescriptor, "min_t", _("Min. T"),  "°C", "%.2f")
	addGridColNumberDescriptor(boundaryGroupTFAnalysisDescriptor, "max_t", _("Max. T"),  "°C", "%.2f")
	addGridColNumberDescriptor(boundaryGroupTFAnalysisDescriptor, "av_t", _("Av. T"),  "°C", "%.2f")
	addGridColNumberDescriptor(boundaryGroupTFAnalysisDescriptor, "Q", "Q",  flowUnit, "%.4f")
	if boundaryVariant != "flows":
		addGridColNumberDescriptor(boundaryGroupTFAnalysisDescriptor, "dew_" + dimensionBind, _("Surf. Cond."),  dimensionUnit, dimensionGridFormat)
	if boundaryVariant == "mold":
		addGridColNumberDescriptor(boundaryGroupTFAnalysisDescriptor, "mold_" + dimensionBind, _("Mold"),  dimensionUnit, dimensionGridFormat)
	
	return boundaryGroupTFAnalysisDescriptor
	
def computeBoundaryGroupTFAnalysis(boundaryGroups, boundaryTFAnalysis, boundaryGroupNames):
	boundaryGroupTFAnalysis = list()

	groupID = 0
	for boundaryGroup in boundaryGroups:
		groupLength = 0.0
		groupMinT = 10000.0
		groupMaxT = -10000.0
		groupAvT = 0.0
		groupQ = 0.0
		groupDewLength = 0.0
		groupMoldLength = 0.0
		for boundary in boundaryGroup:
			boundaryID = int(boundary)
			dimensionBind =  getTFAnalysisDimensionBind(boundaryTFAnalysis[boundaryID])

			groupLength = groupLength + boundaryTFAnalysis[boundaryID][dimensionBind]
			boundaryMinT = boundaryTFAnalysis[boundaryID]["min_t"]
			boundaryMaxT = boundaryTFAnalysis[boundaryID]["max_t"]		
			if boundaryMinT < groupMinT:
				groupMinT = boundaryMinT
			if boundaryMaxT > groupMaxT:
				groupMaxT = boundaryMaxT
			groupAvT = groupAvT + boundaryTFAnalysis[boundaryID]["av_t"] * boundaryTFAnalysis[boundaryID][dimensionBind]
			groupQ = groupQ + boundaryTFAnalysis[boundaryID]["Q"]
			groupDewLength = groupDewLength + boundaryTFAnalysis[boundaryID]["dew_" + dimensionBind] 
			groupMoldLength = groupMoldLength + boundaryTFAnalysis[boundaryID]["mold_" + dimensionBind] 
		if groupLength > 0.0:
			groupAvT = groupAvT / groupLength
		groupAnalysis = dict()
		groupAnalysis["name"] = boundaryGroupNames[groupID]
		groupAnalysis[dimensionBind] = groupLength
		groupAnalysis["min_t"] = groupMinT
		groupAnalysis["max_t"] = groupMaxT
		groupAnalysis["av_t"] = groupAvT
		groupAnalysis["Q"] = groupQ
		groupAnalysis["dew_" + dimensionBind] = groupDewLength
		groupAnalysis["mold_" + dimensionBind] = groupMoldLength
		boundaryGroupTFAnalysis.append(groupAnalysis)
		groupID = groupID + 1
		
	return boundaryGroupTFAnalysis

# Add boundary group analysis to a simulation
def addBoundaryGroupAnalysis(simulation, simulationDescriptors):
	simulation_3d = "3d_simulation" in simulation and simulation["3d_simulation"]

	if "advanced_results" in simulation:
		advancedResults = simulation["advanced_results"]
		advancedResultsDescriptors = findSubDescriptors("advanced_results", simulationDescriptors)
		boundaryResults = advancedResults["boundary"]
		boundaryResultsDescriptors = findSubDescriptors("boundary",  advancedResultsDescriptors)
		boundaryConditions = boundaryResults["boundaries_complete"]
		boundaryTFAnalysis = boundaryResults["tf_analysis"]

		boundaryGroups = boundaryResults["groups"]		
		boundaryGroupNames = generateBoundaryGroupsNames(boundaryGroups, boundaryConditions, "group")
		
		analysisBoundaryGroups = boundaryResults["analysis_groups"];
		analysisBoundaryGroupNames = generateBoundaryGroupsNames(analysisBoundaryGroups, boundaryConditions, "analysis_group")
		boundaryGroups.extend(analysisBoundaryGroups)
		boundaryGroupNames.extend(analysisBoundaryGroupNames)
		
		unaryGroups = True
		for boundaryGroup in boundaryGroups:
			if len(boundaryGroup) != 1:
				unaryGroups = False
				
		boundaryVariant = simulation["boundary_variant"]

		# Add group analysis only if necessary
		if not unaryGroups:
			bind = "group_tf_analysis"
			boundaryResultsDescriptors.insert(findDescriptor("tf_analysis",  boundaryResultsDescriptors) + 1,\
												createBoundaryGroupTFAnalysisDescriptor(bind, boundaryVariant, simulation_3d))
			boundaryResults[bind] = computeBoundaryGroupTFAnalysis(boundaryGroups, boundaryTFAnalysis, boundaryGroupNames)

def computeFRsi(intT, \
				extT, \
				deltaT):
	fRsi = 0.0;
	if deltaT != 0.0:
		fRsi = (intT - extT) / deltaT
		if fRsi > 1.0:
			fRsi = 1.0
		#elif fRsi < 0.0:
		#	fRsi = 0.0
	else:
		if intT - extT > 0.0:
			fRsi = 1.0
		else:
			fRsi = 0.0
	return fRsi;
	
def addAnalysisFRsiComputation(analysis, \
								name, \
								simulation, \
								simulationDescriptors, \
								addFRsiMin):
								
	advancedResults = simulation["advanced_results"]
	boundaryResults = advancedResults["boundary"]
	boundariesAtSimulationTime = boundaryResults["boundaries_at_simulation_time"]
	advancedResultsDescriptors = findSubDescriptors("advanced_results", simulationDescriptors)
	boundaryDescriptors = findSubDescriptors("boundary", advancedResultsDescriptors)
	
	intExtT = computeIntExtT(boundaryResults, boundaryResults["groups"])
	
	fRsiMinMax = 0.0
	minSimFRsi = 1000.0
	#minSimFRsi = -1000.0
	minTsi = 1000.0
	minAcceptableTsi = -1000.0
	internalPhiSat = 1000.0
	for boundaryCouplingIDs in boundaryResults["monthly_couplings_ids"]:
		internalID = boundaryCouplingIDs["internal"]
		externalID = boundaryCouplingIDs["external"]
				
		if internalID < 0 or internalID >= len(analysis) or \
			externalID < 0 or externalID >= len(analysis):
			continue
			
		if (not "length" in analysis[internalID] or analysis[internalID]["length"] <= 0.0) and \
			(not "area" in analysis[internalID] or analysis[internalID]["area"] <= 0.0):
			continue
			
		internalMinT = analysis[internalID]["min_t"]

		if internalMinT < minTsi:
			minTsi = internalMinT
			
		internalT = boundariesAtSimulationTime[internalID]["temperature"]
		internalH = boundariesAtSimulationTime[internalID]["humidity"]
		if(boundariesAtSimulationTime[internalID]["phi_sat"] < internalPhiSat):
			internalPhiSat = boundariesAtSimulationTime[internalID]["phi_sat"]
	#	externalT = boundariesAtSimulationTime[externalID]["temperature"]
		# We're using main couple's external T for simulated fRsi because otherwise
		# we'd get completely wrong results when evaluating condensation risks with 3
		# environments
		externalT = intExtT[1]
		minMoldT = boundariesAtSimulationTime[internalID]["tis_min_mold"]
		if minMoldT > minAcceptableTsi:
			minAcceptableTsi = minMoldT

		deltaT = internalT - externalT

		fRsiMin = computeFRsi(minMoldT, externalT, deltaT)
			
		if fRsiMin > fRsiMinMax:
			fRsiMinMax = fRsiMin

		fRsi = computeFRsi(internalMinT, externalT, deltaT)

		if fRsi != 0.0 and fRsi < minSimFRsi:
			minSimFRsi = fRsi
			
	if name != "":
		bindPostfix = "_" + name
		namePostfix = " " + name
	else:
		bindPostfix = ""
		namePostfix = ""
							
	if minTsi < 1000.0:
		simulationDescriptors.insert(0, createNumberDescriptor("tsi_min" + bindPostfix, \
																_("Simulated Tsi,min") + namePostfix, \
																"°C", \
																"%.2f", \
																"tsi_min" + bindPostfix))
																
		simulation["tsi_min" + bindPostfix] = minTsi
		
		variantPostfix = ""
		if simulation["boundary_variant"] == "mold":
			variantPostfix = " " + _("without mold")
		elif simulation["boundary_variant"] == "dew":
			variantPostfix = " " + _("without cond.")
			
		if name == "":
			rhiDescriptors = simulationDescriptors
			rhiDescriptorPosition = 1
			rhiResults = simulation
			rhiResultDescriptorCenterHorizontal = False
			rhiResultDescriptorSize = list()
			rhiResultDescriptorLabelSize = list()
			rhiResultDescriptorSpan = list()
		else:
			rhiDescriptors = boundaryDescriptors
			rhiDescriptorPosition = 5
			rhiResults = boundaryResults
			rhiResultDescriptorCenterHorizontal = True
			rhiResultDescriptorSize = [600, -1]
			rhiResultDescriptorLabelSize = [200, -1]
			rhiResultDescriptorSpan = [1, 4]
			rhiDescriptors.insert(rhiDescriptorPosition - 1, createSeparatorDescriptor("rhi_separator" + bindPostfix, [1, 4]))

		#rhiDescriptors.insert(rhiDescriptorPosition, createNumberDescriptor( \
		#						"te_min" + bindPostfix, \
		#						_("Te,min") + namePostfix + variantPostfix, \
		#						"°C", \
		#						"%.2f", \
		#						"tse_min" + bindPostfix, \
		#						True, \
		#						"", \
		#						rhiResultDescriptorCenterHorizontal, \
		#						rhiResultDescriptorSize,
		#						rhiResultDescriptorLabelSize,
		#						rhiResultDescriptorSpan))
		
		if 1.0 - minSimFRsi != 0.0:
			simulationDescriptors.insert(1, createNumberDescriptor( \
										"te_min" + bindPostfix, \
										_("Te,min") + namePostfix + variantPostfix, \
										"°C", \
										"%.2f", \
										"tse_min" + bindPostfix))
			
			minTse = (minAcceptableTsi - internalT * minSimFRsi) / (1.0 - minSimFRsi)
			#print("********");
			#print("minTse: " + str(minTse));
			#print("minAcceptableTsi: " + str(minAcceptableTsi));
			#print("internalT: " + str(internalT));
			#print("minSimFRsi: " + str(minSimFRsi));
			rhiResults["te_min" + bindPostfix] = minTse
			simulation["te_min" + bindPostfix] = minTse
								
		rhiDescriptors.insert(rhiDescriptorPosition, createNumberDescriptor( \
								"rhi_min" + bindPostfix, \
								_("Simulated RHsi,min") + "({0:.1f}%)".format(internalH * 100.0) + namePostfix, \
								"%", \
								"%.1f", \
								"hi_min" + bindPostfix, \
								True, \
								"", \
								rhiResultDescriptorCenterHorizontal, \
								rhiResultDescriptorSize,
								rhiResultDescriptorLabelSize,
								rhiResultDescriptorSpan))
		
		#print("internal surf.T " + str(minTsi))
		#print("internalH " + str(internalH))
		#print("psat(minTsi) " + str(psat(minTsi)))
		#print("internalPhiSat " + str(internalPhiSat))
		#print("Pv sat limite " + str(psat(minTsi) * internalPhiSat))
		#print("internalT " + str(internalT))
		#print("minPv " + str(psat(minTsi) * internalPhiSat))
		#print("psat(internalT) " + str(psat(internalT)))
		#print("internalH,max " + str(psat(minTsi) * internalPhiSat / psat(internalT)))
		#print("internalPv " + str(internalH * psat(internalT)))
								
		#minHi = internalH * psat(internalT) / (psat(minTsi) * internalPhiSat)
		minHi = internalH * psat(internalT) / (psat(minTsi))
		rhiResults["rhi_min" + bindPostfix] = min(1.0, minHi)
		
		rhiDescriptors.insert(rhiDescriptorPosition + 1, createNumberDescriptor("rhi_max" + bindPostfix, \
								_("Internal H,max") + namePostfix + variantPostfix, \
								"%", \
								"%.1f", \
								"hi_max" + bindPostfix, \
								True, \
								"", \
								rhiResultDescriptorCenterHorizontal, \
								rhiResultDescriptorSize,
								rhiResultDescriptorLabelSize,
								rhiResultDescriptorSpan))
								
		maxHi = psat(minTsi) * internalPhiSat / psat(internalT)
		rhiResults["rhi_max" + bindPostfix] = min(1.0, maxHi)
		
	if fRsiMinMax > 0.0 and addFRsiMin:
		simulationDescriptors.insert(0, createNumberDescriptor("frsi_min" + bindPostfix, \
																_("fRsi,min") + namePostfix, \
																"", \
																"%.3f", \
																"min_frsi" + bindPostfix))
		simulation["frsi_min" + bindPostfix] = fRsiMinMax			
	if minSimFRsi < 1000.0:
		simulationDescriptors.insert(0, createNumberDescriptor("min_sim_frsi" + bindPostfix, \
																_("Min simulated fRsi") + namePostfix, \
																"", \
																"%.3f", \
																"min_sim_frsi" + bindPostfix))
		simulation["min_sim_frsi" + bindPostfix] = minSimFRsi			
			
# Add fRsi computation to a simulation
def addFRsiComputation(simulation, simulationDescriptors):
	if "advanced_results" in simulation:
		advancedResults = simulation["advanced_results"]
		boundaryResults = advancedResults["boundary"]
		boundaryTFAnalysis = boundaryResults["tf_analysis"]
		
		if("masked_tf_analysis" in boundaryResults):
			maskedBoundaryTFAnalysis = boundaryResults["masked_tf_analysis"]
			for analysisName, analysis in maskedBoundaryTFAnalysis.items():
				addAnalysisFRsiComputation(analysis, \
											analysisName, \
											simulation, \
											simulationDescriptors, \
											False)
											
		addAnalysisFRsiComputation(boundaryTFAnalysis, \
									"", \
									simulation, \
									simulationDescriptors, \
									True)
									
def combineAnalysis(analysis1, analysis2):
	dimensionBind = getTFAnalysisDimensionBind(analysis1)

	if analysis2["min_t"] < analysis1["min_t"]:
		analysis1["min_t"] = analysis2["min_t"]
		analysis1["min_t_pos"] = analysis2["min_t_pos"]
	if analysis2["max_t"] > analysis1["max_t"]:
		analysis1["max_t"] = analysis2["max_t"]
	if analysis2["min_flux"] < analysis1["min_flux"]:
		analysis1["min_flux"] = analysis2["min_flux"]
	if analysis2["max_flux"] > analysis1["max_flux"]:
		analysis1["max_flux"] = analysis2["max_flux"]
	analysis1["av_t"] += analysis2["av_t"]
	analysis1["av_flux"] += analysis2["av_flux"]
	analysis1["dew_" + dimensionBind] += analysis2["dew_" + dimensionBind]
	analysis1["mold_" + dimensionBind] += analysis2["mold_" + dimensionBind]
	
def endCombiningAnalysis(analysis, steps):
	dimensionBind = getTFAnalysisDimensionBind(analysis)

	analysis["av_t"] = analysis["av_t"] / steps
	analysis["av_flux"] = analysis["av_flux"] / steps
	analysis["dew_" + dimensionBind] = analysis["dew_" + dimensionBind] / steps
	analysis["mold_" + dimensionBind] = analysis["mold_" + dimensionBind] / steps

def removeUnusedBoundariesAnalysis(simulation):
	if "advanced_results" in simulation:
		advancedResults = simulation["advanced_results"]
		boundaryResults = advancedResults["boundary"]
		boundaryTFAnalysis = boundaryResults["tf_analysis"]
		
		for boundaryID in reversed(range(len(boundaryTFAnalysis))):
			if ("length" not in boundaryTFAnalysis[boundaryID] or boundaryTFAnalysis[boundaryID]["length"] == 0.0) and \
				("area" not in boundaryTFAnalysis[boundaryID] or boundaryTFAnalysis[boundaryID]["area"] == 0.0):
				boundaryTFAnalysis.pop(boundaryID)
				
def add10077BoundaryResults(boundaryResults,\
							boundaryGroups,\
							biraSettings,\
							output,\
							outputDescriptors):
	boundaries = boundaryResults["boundaries_complete"]
	internalBoundaryID = -1
	internalIncBoundaryID = -1
	externalBoundaryID = -1
	
	if "boundary_to_increase" in biraSettings and \
		"increased_boundary" in biraSettings and \
		"external_boundary_to_increase" in biraSettings:
		internalBoundaryID = biraSettings["boundary_to_increase"]
		internalIncBoundaryID = biraSettings["increased_boundary"]
		externalBoundaryID = biraSettings["external_boundary_to_increase"]
		
	if internalBoundaryID < 0 or internalBoundaryID >= len(boundaries) or \
		internalIncBoundaryID < 0 or internalIncBoundaryID >= len(boundaries) or \
		externalBoundaryID < 0 or externalBoundaryID >= len(boundaries):
		minT = 100000
		maxT = -100000
		for boundaryGroup in boundaryGroups:
			for boundaryID in boundaryGroup:
				boundary = boundaries[boundaryID]
				if(boundary["type"] != "environment_air"):
					continue
				airTemperature = boundary["air_temperature"]
				if(airTemperature < minT):
					minT = airTemperature
				if(airTemperature > maxT):
					maxT = airTemperature
					
		internalR = 0	
		for boundaryGroup in boundaryGroups:
			for boundaryID in boundaryGroup:
				boundary = boundaries[boundaryID]
				if(boundary["type"] != "environment_air"):
					continue

				airTemperature = boundary["air_temperature"]
				surfaceResistance = boundary["surface_resistance"]
				
				if(airTemperature == minT):
					externalBoundaryID = boundaryID
				if(airTemperature == maxT):
					if(internalBoundaryID == -1):
						internalBoundaryID = boundaryID
						internalR = surfaceResistance
					else:
						if(surfaceResistance < internalR):
							internalIncBoundaryID = internalBoundaryID
							internalBoundaryID = boundaryID
						else:
							internalIncBoundaryID = boundaryID
				
	if(externalBoundaryID != -1 and internalBoundaryID != -1):
		outputDescriptors.insert(0, createNumberDescriptor("delta_t", _("ΔT"), "°C", "%.2f", "delta_t"))
		output["delta_t"] = math.fabs(boundaries[internalBoundaryID]["air_temperature"] - boundaries[externalBoundaryID]["air_temperature"])
	if(externalBoundaryID != -1):
		outputDescriptors.insert(0, createNumberDescriptor("external_t", _("External temperature"), "°C", "%.2f", "ex_t"))
		output["external_t"] = boundaries[externalBoundaryID]["air_temperature"]
		outputDescriptors.insert(0, createNumberDescriptor("external_r", _("External surface resistance"), "m²K/W", "%.3f", "ex_r", False))
		output["external_r"] = boundaries[externalBoundaryID]["surface_resistance"]
	if(internalBoundaryID != -1):
		outputDescriptors.insert(0, createNumberDescriptor("internal_t", _("Internal temperature"), "°C", "%.2f", "in_t"))
		output["internal_t"] = boundaries[internalBoundaryID]["air_temperature"]
		outputDescriptors.insert(0, createNumberDescriptor("internal_r", _("Internal surface resistance"), "m²K/W", "%.3f", "in_r", False))
		output["internal_r"] = boundaries[internalBoundaryID]["surface_resistance"]
		if "humidity" in boundaries[internalBoundaryID]:
			outputDescriptors.insert(0, createNumberDescriptor("internal_h", _("Internal surface resistance"), "%", "%.2f", "in_hum", False))
			output["internal_h"] = boundaries[internalBoundaryID]["humidity"]
			
	if(internalIncBoundaryID != -1):
		outputDescriptors.insert(0, createNumberDescriptor("internal_inc_r", _("Int. inc. surface resistance"), "m²K/W", "%.3f", "in_inc_r", False))
		output["internal_inc_r"] = boundaries[internalIncBoundaryID]["surface_resistance"]
			