import math
import imp
import time
imp.load_source("util","ScriptEditor/util.py") 
from util import *
imp.load_source("lgFactors","ScriptEditor/lgFactors.py") 
from lgFactors import *
imp.load_source("l2dResults","ScriptEditor/l2dResults.py") 
from l2dResults import *
imp.load_source("boundary","ScriptEditor/boundary.py") 
from boundary import *
imp.load_source("dynamic","ScriptEditor/dynamic.py") 
from dynamic import *
imp.load_source("dynamicLag","ScriptEditor/dynamicLag.py") 
from dynamicLag import *
imp.load_source("boundary","ScriptEditor/simulationResults.py") 
from boundary import *


currentGeometry = 0
currentFill = 0
currentBoundary = 0

startTime = time.time()

if "default_geometry_id" in Input:
	currentGeometry = Input["default_geometry_id"]

simulationSettings = Simulator.getSimulationSettings(currentGeometry, currentFill, currentBoundary)
	
Output = dict()
OutputDescriptors = list()

geometryInfo = Simulator.getGeometryInfo()
specialElementsInfo = Simulator.getSpecialElementsInfo(currentGeometry)
fillInfo = Simulator.getFillInfo(currentGeometry)
boundaryInfo = Simulator.getBoundaryInfo(currentGeometry, currentFill)
boundaryConditions = Simulator.getBoundaryConditions(currentGeometry, currentFill, currentBoundary)
boundaryTiming = Simulator.getBoundaryTiming(currentGeometry, currentFill, currentBoundary)

specialElementsConfig = list()
if specialElementsInfo[0]["element"]["__descriptor_name__"] == "__bridge_section_elements__":
	# Just tell simulator to get a description of section elements
	specialElementsConfig.append(1)
elif specialElementsInfo[0]["element"]["__descriptor_name__"] == "__10077-2_insulation_panel__":
	# Just tell simulator to get a description of section elements
	specialElementsConfig.append(3)
Input["boundary_variant"] = "flows"
flowSimulation, flowSimulationDescriptors = addSimulation(Simulator,\
														currentGeometry,\
														currentFill,\
														currentBoundary,\
														specialElementsConfig,\
														boundaryConditions,\
														Input,\
														Output,\
														OutputDescriptors,\
														_("Flow simulation"),\
														"flow_s")
addDynamicBoundaryXYZFactorCurves(boundaryConditions, flowSimulation, flowSimulationDescriptors)

bridgeSectionElements = specialElementsInfo[0]["element"]["__descriptor_name__"] == "__bridge_section_elements__"
insulationPanel10077 = specialElementsInfo[0]["element"]["__descriptor_name__"] == "__10077-2_insulation_panel__"
simulation_3d = "3d_simulation" in flowSimulation and flowSimulation["3d_simulation"]

computeDew = False
computeUtjPsij = False
if "flows" in simulationSettings:
	if "compute_dew" in simulationSettings["flows"]:
		computeDew = simulationSettings["flows"]["compute_dew"]
if "simulation_settings" in simulationSettings:		
	if "computer_utj_psij" in simulationSettings["simulation_settings"]:
		computeUtjPsij = simulationSettings["simulation_settings"]["computer_utj_psij"]
		
if computeDew:
	Input["boundary_variant"] = "dew"
	specialElementsConfig = list()
	if bridgeSectionElements:
		# Tell simulator to perform analysis on section elements
		specialElementsConfig.append(2)
	elif insulationPanel10077:
		specialElementsConfig.append(0)
	dewSimulation, dewSimulationDescriptors = addSimulation(Simulator,\
															currentGeometry,\
															currentFill,\
															currentBoundary,\
															specialElementsConfig,\
															boundaryConditions,\
															Input,\
															Output,\
															OutputDescriptors,\
															_("Surf. Cond."),\
															"dew_s")
	addDynamicBoundaryXYZFactorCurves(boundaryConditions, dewSimulation, dewSimulationDescriptors)														

computeMold = False
if "flows" in simulationSettings:
	if "compute_mold" in simulationSettings["flows"]:
		computeMold = simulationSettings["flows"]["compute_mold"]
if computeMold:
	Input["boundary_variant"] = "mold"
	specialElementsConfig = list()
	if bridgeSectionElements:
		# Tell simulator to perform analysis on section elements
		specialElementsConfig.append(2)
	elif insulationPanel10077:
		specialElementsConfig.append(0)
	moldSimulation, moldSimulationDescriptors = addSimulation(Simulator,\
																currentGeometry,\
																currentFill,\
																currentBoundary,\
																specialElementsConfig,\
																boundaryConditions,\
																Input,\
																Output,\
																OutputDescriptors,\
																_("Mold"),\
																"cond_s")
	addDynamicBoundaryXYZFactorCurves(boundaryConditions, moldSimulation, moldSimulationDescriptors)														

panelSimulation10077 = dict()
panelSimulation10077Descriptors = list()

performPanelSimulation = insulationPanel10077 and specialElementsInfo[0]["num_cycles"] > 0
if performPanelSimulation:
	# Perform a simulation substituting delimited areas with an insulation panel
	specialElementsConfig = list()
	specialElementsConfig.append(1)
	Input["boundary_variant"] = "flows"
	panelSimulation10077, panelSimulation10077Descriptors = addSimulation(Simulator,\
																		currentGeometry,\
																		currentFill,\
																		currentBoundary,\
																		specialElementsConfig,\
																		boundaryConditions,\
																		Input,\
																		Output,\
																		OutputDescriptors,\
																		_("10077 Insulation panel"),\
																		"10077_ip_s")

	#anelSimulation10077Descriptors.insert(0, createNumberDescriptor("heat_flow", _("Flow"), "W/m", "%.4f"))
	#panelSimulation10077["heat_flow"] = getAvFlow(panelSimulation10077["advanced_results"]["boundary"]["tf_analysis"])
	#flowSimulationDescriptors.insert(0, createNumberDescriptor("heat_flow", _("Flow"), "W/m", "%.4f"))
	#flowSimulation["heat_flow"] = getAvFlow(flowSimulation["advanced_results"]["boundary"]["tf_analysis"])
	
	addDynamicBoundaryXYZFactorCurves(boundaryConditions, panelSimulation10077, panelSimulation10077Descriptors)														
else:
	panelSimulation10077 = flowSimulation
	panelSimulation10077Descriptors = list()
	#panelSimulation10077["heat_flow"] = getAvFlow(panelSimulation10077["advanced_results"]["boundary"]["tf_analysis"])

advancedResults = flowSimulation["advanced_results"]
advancedResultsDescriptors = findSubDescriptors("advanced_results", flowSimulationDescriptors)

boundaryResults = advancedResults["boundary"]
boundaryResultsDescriptors = findSubDescriptors("boundary", advancedResultsDescriptors)

# Get temperatures to be used for L-G computation
boundaryGroups = boundaryResults["groups"]
lgAirTemperature1, lgAirTemperature2 = getLGAirTemperatures(boundaryConditions, boundaryGroups)

# Generate groups name
boundaryGroupNames = generateBoundaryGroupsNames(boundaryGroups, boundaryConditions)

computeL = False
computeG = False
if "flows" in simulationSettings:
	computeL = simulationSettings["flows"]["compute_l"]
	computeG = simulationSettings["flows"]["compute_g"] and computeMold
	
deltaT = computeDeltaT(boundaryResults, boundaryGroups)
	
if computeL:
	boundaryResultsDescriptors.append(createLResultsTableDescriptor(boundaryGroups, simulation_3d))

	# L results
	lComputationsGrid = generateLResultsGrid(len(boundaryGroups))
	boundaryResults["l_computations"] = computeLResultsTable(Simulator,\
															currentGeometry,\
															currentFill,\
															currentBoundary,\
															flowSimulation,\
															Input,\
															Output,\
															OutputDescriptors,\
															lgAirTemperature1,\
															lgAirTemperature2,\
															lComputationsGrid,\
															boundaryGroups)

	# Create L  table
	boundaryResultsDescriptors.append(createLTableDescriptor(boundaryGroupNames, simulation_3d))
	boundaryResults["l_table"] = computeLTable(boundaryGroupNames, boundaryResults["l_computations"])

	# Create flow table
	boundaryResultsDescriptors.append(createFlowTableDescriptor(boundaryGroupNames, simulation_3d))
	boundaryResults["flow_table"] = computeFlowsTable(boundaryResults, boundaryConditions, boundaryGroups)

	# L2D computations
	bridgedL2D, psi, additionalPsi = computeBridgedL2D(boundaryResults["l_computations"],\
														boundaryResults["l_table"],\
														boundaryResults["flow_table"],\
														boundaryGroups,\
														advancedResults)
else:
	bridgedL2D = computeBridgedL2DNoL(boundaryResults, deltaT)
	psi = [0.0]
	additionalPsi = list()
	
if computeG:
	moldSimulationAdvanced = moldSimulation["advanced_results"]
	moldSimulationBoundary = moldSimulationAdvanced["boundary"]
	moldSimulationAdvancedDescriptors = findSubDescriptors("advanced_results",  moldSimulationDescriptors)
	moldSimulationBoundaryDescriptors = findSubDescriptors("boundary",  moldSimulationAdvancedDescriptors)
	
	moldSimulationBoundaryDescriptors.append(createGTableDescriptor(boundaryGroupNames))
	moldSimulationBoundary["g_table"] = computeGTable(Simulator,\
																currentGeometry,\
																currentFill,\
																currentBoundary,\
																moldSimulation,\
																Input,\
																Output,\
																OutputDescriptors,\
																boundaryGroups,\
																boundaryGroupNames)
																
if insulationPanel10077:
	noBridgeL2D, additionalResults = computeNoBridgeL2D10077(boundaryGroups,\
															advancedResults,\
															panelSimulation10077,\
															bridgedL2D,\
															deltaT,\
															computeUtjPsij)
	
	biraSettings = Simulator.getSettings(currentGeometry, currentFill, currentBoundary, "boundary_increased_resistance")															
	add10077BoundaryResults(boundaryResults,\
							boundaryGroups,\
							biraSettings,\
							Output,\
							OutputDescriptors)
							
	add10077UfResults(bridgedL2D,\
						additionalResults,\
						Output,\
						OutputDescriptors,\
						flowSimulation,\
						flowSimulationDescriptors,\
						panelSimulation10077,\
						panelSimulation10077Descriptors)
else:
	noBridgeL2D = computeNoBridgeL2D(deltaT,\
										advancedResults,\
										Simulator,\
										currentGeometry,\
										currentFill,\
										currentBoundary,\
										Input,\
										Output,\
										OutputDescriptors)
										
	if simulation_3d:
		noBridgeL1D_3D = computeNoBridgeL1D_3D(advancedResults,\
											Simulator,\
											currentGeometry)
		noBridgeL2D_3D = computeNoBridgeL2D_3D(advancedResults,\
											Simulator,\
											currentGeometry)
		addL1DResults_3D(noBridgeL1D_3D,
						flowSimulation,\
						flowSimulationDescriptors)
		addL2DResults_3D(noBridgeL2D_3D,
						flowSimulation,\
						flowSimulationDescriptors)
		addL3DResults_3D(bridgedL2D,\
						flowSimulation,\
						flowSimulationDescriptors)
		addPSIResults_3D(bridgedL2D,\
						noBridgeL1D_3D,\
						noBridgeL2D_3D,\
						flowSimulation,\
						flowSimulationDescriptors)												

if not simulation_3d:
	addUResults(bridgedL2D,\
				flowSimulation,\
				flowSimulation,\
				flowSimulationDescriptors)
	if not insulationPanel10077:
		addFlowResults(bridgedL2D,\
						noBridgeL2D,\
						psi,\
						deltaT,\
						flowSimulation,\
						flowSimulationDescriptors)
		addL2DResults(bridgedL2D,\
						noBridgeL2D,\
						psi,\
						additionalPsi,\
						flowSimulation,\
						flowSimulation,\
						flowSimulationDescriptors,\
						insulationPanel10077)
	else:
		addL2DResults(bridgedL2D,\
						noBridgeL2D,\
						psi,\
						additionalPsi,\
						flowSimulation,\
						Output,\
						OutputDescriptors,\
						insulationPanel10077)

	if("l2d_bridged" in Output and bridgedL2D == 0.0):
		Output["error"] = _("Bridged L2D is zero. Please check your project")
	
if "transient" in simulationSettings and simulationSettings["transient"]["compute_lag"]:
	computeLag(Simulator,\
				currentGeometry,\
				currentFill,\
				currentBoundary,\
				flowSimulation,\
				flowSimulationDescriptors,\
				Input,\
				Output,\
				OutputDescriptors,\
				boundaryGroups)

if "dynamic_timing" in boundaryTiming and boundaryTiming["dynamic_timing"]["active"]:
	computeDynamic(Simulator,\
				currentGeometry,\
				currentFill,\
				currentBoundary,\
				flowSimulation,\
				flowSimulationDescriptors,\
				Input,\
				Output,\
				OutputDescriptors,\
				boundaryGroups,\
				boundaryTiming["dynamic_timing"]["__descriptor_name__"],\
				boundaryTiming["dynamic_timing"]["start_time"],\
				boundaryTiming["dynamic_timing"]["end_time"],\
				boundaryTiming["dynamic_timing"]["static_simulation_offset"])
				
addSimulationTimeResult(startTime, OutputDescriptors, Output)

pslgBoundarySettings = Simulator.getSettings(currentGeometry, currentFill, currentBoundary, "pslg_boundary")
pslgBoundaryDescriptors = Simulator.getSettings(currentGeometry, currentFill, currentBoundary, "pslg_boundary_descriptors")
addCavityDetectionResult(pslgBoundarySettings, \
						pslgBoundaryDescriptors, \
						OutputDescriptors, \
						Output)
						